/*****************************************************************************
*  Copyright 2004 - 2007 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

/*
 * u-boot/common/cmd_sdhci.c
 * SD/SDHC/MMC commands.
 */
#include <common.h>
#include <asm/arch/bcm4760.h>
#include <sdhci.h>
#include <asm/arch/sdhci.h>

extern void sdhci_display_capabilites(int dev_num);


#ifdef CONFIG_CMD_SDHCI

/*****************************************************************************
 * Boot Strap
 *
 * This should be removed when stage_I starts filling in the boot device
 */
//#define BOOT_STRAP_PATCH


/*****************************************************************************
 * Boot Device Info
 *
 * This index will describe the device to retrieve u-boot from
 *		BOOTIF_SDIOA_IDX
 *		BOOTIF_SDIOB_IDX
 *		BOOTIF_SDIOC_IDX
 *		BOOTIF_SFLASH_IDX
 *		BOOTIF_NAND_IDX
 *		BOOTIF_UART_DL_IDX
 */
#define	BOOT_DEVICE_IDX_ADDR	((volatile unsigned long*)(0x0020FFF4))


/* ------------------------------------------------------------------------- */

#ifndef TRUE
#define		TRUE			1
#endif
#ifndef FALSE
#define		FALSE			0
#endif

#define		BLOCK_SIZE		512

static int sd_inited = 0;
static int sd_device = -1;
static ulong sd_read_buf[(BLOCK_SIZE/sizeof(ulong))];

/* ------------------------------------------------------------------------- */


/* ------------------------------------------------------------------------- */
static void sd_usage(void)
{
	puts("sd - SD/SDHC/MMC sub-system\n");
	puts("sd info - show available devices\n");
	puts("sd info <dev> - show details of device <dev>\n");
	puts("sd reset - reset all devices\n");
	puts("sd reset <dev> - reset device <dev>\n");
	puts("sd device - show current device\n");
	puts("sd device <dev> - set <dev> as the current device\n");
	puts("sd read <blk#> <cnt> - read <cnt> blocks starting at <blk#>\n");
	puts("sd write <addr> <blk#> <cnt> - write <cnt> blocks starting\n");
	puts("    at <blk#> from memory address <addr>\n");
	puts("sd boot\n");
}

void print_sd_info(char* str, int dev, int capable)
{
	block_dev_desc_t* dev_desc = sdhci_get_dev(dev);

	if (str)
	{
		printf(str, dev);
	}
	if (((dev >= 0) && (dev < CFG_SDHCI_MAXDEVICE)) &&
		(dev_desc->type != DEV_TYPE_UNKNOWN))
	{
		switch (dev_desc->if_type)
		{
			case IF_TYPE_SD:
			{
				puts("SD");
			}
			break;
#if 0
            case IF_TYPE_SDHC:
            {
                 puts("SDHC");
            }
            break;
#endif
			case IF_TYPE_MMC:
			{
				puts("MMC");
			}
			break;
		}
		puts(" Memory\n");
		printf("    Vendor: %s\n",
			   dev_desc->vendor);
		printf("    Product: %s Revision: %s\n", 
			   dev_desc->product,
			   dev_desc->revision);

		if ((dev_desc->lba * dev_desc->blksz)>0L)
		{
#ifdef CONFIG_LBA48
			uint64_t lba512; /* number of blocks if 512bytes block size */
#else
			lbaint_t lba512;
#endif
			ulong mb, mb_quot, mb_rem, gb, gb_quot, gb_rem;
			lbaint_t lba;

			lba = dev_desc->lba;

			lba512 = (lba * (dev_desc->blksz/512));
			mb = (10 * lba512) / 2048;	/* 2048 = (1024 * 1024) / 512 MB */
			/* round to 1 digit */
			mb_quot	= mb / 10;
			mb_rem	= mb - (10 * mb_quot);

			gb = mb / 1024;
			gb_quot	= gb / 10;
			gb_rem	= gb - (10 * gb_quot);
#ifdef CONFIG_LBA48
			if (dev_desc->lba48)
				printf ("            Supports 48-bit addressing\n");
#endif
#if defined(CFG_64BIT_LBA) && defined(CFG_64BIT_VSPRINTF)
			printf("    Capacity: %ld.%ld MB = %ld.%ld GB (%qd x %ld)\n",
				   mb_quot, mb_rem,
				   gb_quot, gb_rem,
				   lba,
				   dev_desc->blksz);
#else
			printf("    Capacity: %ld.%ld MB = %ld.%ld GB (%ld x %ld)\n",
				   mb_quot, mb_rem,
				   gb_quot, gb_rem,
				   (ulong)lba,
				   dev_desc->blksz);
#endif
		}
		else
		{
			puts("            Capacity: not available\n");
		}

		if (capable)
		{
			sdhci_display_capabilites(dev);
		}
	}
	else
	{
		puts("Not Available\n");
	}
}

int do_sd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int dev;
	int rcode = 0;
	int did_init = 0;

#ifdef CONFIG_CMD_FAT
	/*
	 * Hack by Broadcom (TM) to speed up boot
	 */
	if (argc == 2 && strncmp(argv[1],"boo",3) == 0)
	{
		extern int do_bootm(cmd_tbl_t *, int, int, char *[]);
		extern int do_fat_fsload(cmd_tbl_t *, int, int, char *[]);

		char  bootdev[4];
		char* bootfile;
		char* loadaddr;
		char* rootfsfile;
		char* rootfsloadaddr;

		char* largv[ 5];

		dev = *BOOT_DEVICE_IDX_ADDR;
#ifdef BOOT_STRAP_PATCH
		/**
		 ** PATCH in the boot device from the straps if it is
		 ** out of range.  This is currently used for debug as
		 ** stage_II takes care of it if we boot from ROM
		 **/
		{
			uint32_t cmu_ctl0;

			cmu_ctl0 = *((volatile uint32_t*)(CMU_R_CMU_CTL0_MEMADDR));
			dev = CMU_F_BOOTSTRAP_OPTIONS_GET(cmu_ctl0);
		}
#endif
		sprintf(bootdev, "%1d:1", dev);
		sdhci_init(dev);
		sd_inited = 1;
		printf("Trying to boot device sdm(%s)\n", bootdev);

		if (!(loadaddr = getenv("loadaddr")))
		{
			printf("failed - Invalid loadaddr in nvconfig\n");
			return 1;
		}
		if (!(bootfile = getenv("bootfile")))
		{
			printf("failed - Invalid bootfile in nvconfig\n");
			return 1;
		}
		if (!(rootfsloadaddr = getenv("rootfsloadaddr")))
		{
			printf("failed - Invalid rootfsloadaddr in nvconfig\n");
			return 1;
		}
		if (!(rootfsfile = getenv("rootfsfile")))
		{
			printf("failed - Invalid rootfsfile in nvconfig\n");
			return 1;
		}
		/*
		 * Load Linux
		 */
		{
			largv[0] = "fatload";
			largv[1] = "sdm";
			largv[2] = bootdev;
			largv[3] = loadaddr;
			largv[4] = bootfile;
			if (do_fat_fsload(cmdtp, 0, 5, largv))
			{
				printf("failed to load - %s\n", bootfile);
				return 1;
			}
		}

		/*
		 * Load root file system
		 */
		{
			largv[0] = "fatload";
			largv[1] = "sdm";
			largv[2] = bootdev;
			largv[3] = rootfsloadaddr;
			largv[4] = rootfsfile;
			if (do_fat_fsload(cmdtp, 0, 5, largv))
			{
				printf("failed to load - %s\n", rootfsfile);
				return 1;
			}
		}

		/*
		 * Boot
		 */
		{
			largv[0] = "bootm";
			largv[1] = loadaddr;
			if (do_bootm(cmdtp, 0, 2, largv))
			{
				printf("failed to boot\n");
				return 1;
			}
		}
	}
#endif

/*
 *	This code initializes all the MMC channels in the controller
 *	When using sd device xxx we don't need this because we are 
 *	explicitly specifying which device we want initialized
 *	Avoiding this initialization saves 3s in u-boot!
	if (!sd_inited)
	{
		for (dev=0; dev<CFG_SDHCI_MAXDEVICE; ++dev)
		{
			printf ("--- Init device: %d\n", dev);
			sdhci_init(dev);
		}
		did_init = 1;
		sd_inited = 1;
	}
*/
	switch (argc)
	{
		case 2:
		{
			if (strncmp(argv[1],"res",3) == 0)
			{
				for (dev=0; dev<CFG_SDHCI_MAXDEVICE; ++dev)
				{
					sdhci_init(dev);
					print_sd_info("Reset Storage device %d: ", dev, FALSE);
				}
			}
			else if (strncmp(argv[1],"inf",3) == 0)
			{
				for (dev=0; dev<CFG_SDHCI_MAXDEVICE; ++dev)
				{
					print_sd_info("Storage device %d: ", dev, FALSE);
				}
			}
			else if (strncmp(argv[1],"dev",3) == 0)
			{
				if ((sd_device < 0) || (sd_device >= CFG_SDHCI_MAXDEVICE))
				{
					rcode = 1;
					puts("No Storage device selected\n");
				}
				else
				{
					print_sd_info("Selected Storage device %d\n", sd_device, FALSE);
				}
			}
			else
			{
				rcode = 1;
				sd_usage();
			}
		}
		break;

		case 3:
		{
			int dev = (int)simple_strtoul(argv[2], NULL, 10);

			if (strncmp(argv[1],"res",3) == 0)
			{
				sdhci_init(dev);
				print_sd_info("Reset Storage device %d: ", dev, FALSE);
			}
			else if (strncmp(argv[1],"inf",3) == 0)
			{
				print_sd_info("Storage device %d: ", dev, TRUE);
			}
			else if (strncmp(argv[1],"dev",3) == 0)
			{
				printf ("Select Storage device %d\n", dev);
				if (dev >= CFG_SDHCI_MAXDEVICE)
				{
					rcode = 1;
					puts ("unknown device\n");
				}
				else
				{
					block_dev_desc_t* dev_desc = sdhci_get_dev(sd_device);

					if (sd_device != dev)
					{
						sdhci_init(dev);
						sd_device = dev;
					}
					print_sd_info(NULL, dev, FALSE);

					if (dev_desc->type == DEV_TYPE_UNKNOWN)
					{
						sd_device = -1;
						rcode = 1;
					}
					else
					{
						puts("... is now current device\n");
					}
				}
			}
			else
			{
				rcode = 1;
				sd_usage();
			}
		}
		break;

		case 4:
		{
			if (strcmp(argv[1],"read") == 0)
			{
		
#define		DISP_LINE_LEN 	16
		
				ulong cnt = simple_strtoul(argv[3], NULL, 16);
				ulong i, nbytes = BLOCK_SIZE, size = 4, linebytes, n;
				u_char *cp;
				int x;
		
#ifdef CFG_64BIT_STRTOUL
				lbaint_t blk  = simple_strtoull(argv[2], NULL, 16);

				printf("Storage read: device %d block # %qd, count %ld ... \n",
					   sd_device, blk, cnt);
#else
				lbaint_t blk  = simple_strtoul(argv[2], NULL, 16);

				printf("Storage read: device %d block # %ld, count %ld ... \n",
					   sd_device, blk, cnt);
#endif

				for ( x=0; x<cnt; x++)
				{
					ulong* buf_ptr = sd_read_buf;

					n = sdhci_read(sd_device,
								   blk+x,
								   1,
								   sd_read_buf);
					if (n != 1)
					{
						rcode = 1;
						printf("Error");
						break;
					}

					nbytes = BLOCK_SIZE;
					do
					{
						u_char	linebuf[DISP_LINE_LEN];
						uint	*uip = (uint*)linebuf;

						printf("%08lx:", BLOCK_SIZE - nbytes + ((x + blk) * BLOCK_SIZE));
						linebytes = (nbytes>DISP_LINE_LEN)?DISP_LINE_LEN:nbytes;

						for (i=0; i<linebytes; i+= size)
						{
							printf(" %08x", (*uip++ = *((uint*)buf_ptr)));
							buf_ptr += 1;
						}

						puts("    ");
						cp = linebuf;
						for (i=0; i<linebytes; i++)
						{
							if ((*cp < 0x20) || (*cp > 0x7e))
							{
								putc('.');
							}
							else
							{
								printf("%c", *cp);
							}
							cp++;
						}
						putc('\n');
						nbytes -= linebytes;
					} while (nbytes > 0);
				}
			}
			else
			{
				rcode = 1;
				sd_usage();
			}
		}
		break;

		case 5:
		{
			if (strcmp(argv[1],"write") == 0)
			{
				ulong addr = simple_strtoul(argv[2], NULL, 16);
				ulong cnt  = simple_strtoul(argv[4], NULL, 16);
				ulong n;
#ifdef CFG_64BIT_STRTOUL
				lbaint_t blk  = simple_strtoull(argv[3], NULL, 16);

				printf ("Storage write: device %d block # %qd, count %ld ... ",
						sd_device, blk, cnt);
#else
				lbaint_t blk  = simple_strtoul(argv[3], NULL, 16);

				printf("Storage write: device %d block # %ld, count %ld ... ",
					   sd_device, blk, cnt);
#endif

				n = sdhci_write(sd_device,
								blk,
								cnt,
								(ulong*)addr);
				printf("%ld blocks written: %s\n",
					   n, (n==cnt) ? "OK" : "ERROR");
				if (n != cnt)
				{
					rcode = 1;
				}
			}
			else
			{
				rcode = 1;
				sd_usage();
			}
		}
		break;

		default:
		{
			rcode = 1;
			sd_usage();
		}
	}
	return rcode;
}


void sdboot_usage(void)
{
	puts("sdboot - boot from SD/SDHC/MMC raw device\n");
	puts("sdboot <addr> <dev> <blk#> <cnt> - boot <cnt> blocks starting\n");
	puts("    at block <blk#> on device <dev> at address <addr>\n");
}

int do_sdboot (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong addr;
	int dev;
	lbaint_t blk;
	ulong blkcnt;
	char* ep;
	block_dev_desc_t* dev_desc;

	/*
	 * Make sure the SDHCI tables are populated
	 */
	if (!sd_inited)
	{
		for (dev=0; dev<CFG_SDHCI_MAXDEVICE; ++dev)
		{
			sdhci_init(dev);
		}
		sd_inited = 1;
	}

	switch (argc)
	{
		case 5:
		{
			addr = simple_strtoul(argv[1], NULL, 16);
			dev = simple_strtoul(argv[2], NULL, 16);
#ifdef CFG_64BIT_STRTOUL
			blk = simple_strtoull(argv[3], NULL, 16);
#else
			blk = simple_strtoul(argv[3], NULL, 16);
#endif
			blkcnt = simple_strtoul(argv[4], NULL, 16);

			dev_desc = sdhci_get_dev(dev);
		}
		break;

		default:
		{
			sdboot_usage();
		}
		return 1;
	}

	/*
	 * Initialize the device and select it as current
	 */
	sdhci_init(dev);
	print_sd_info("Booting from device %d: ", dev, FALSE);
	sd_device = dev;

	if (dev_desc->type == DEV_TYPE_UNKNOWN)
	{
		printf("** Device %d not available\n", dev);
		return 1;
	}

	if (sdhci_read(dev, blk, blkcnt, (void*)addr) != blkcnt)
	{
		printf("** Read error on Device %d\n", dev);
		return 1;
	}

	/* Loading ok, update default load address */
	load_addr = addr;

	/* Check if we should attempt an auto-start */
	if (((ep = getenv("autostart")) != NULL) && (strcmp(ep,"yes") == 0))
	{
		char *local_args[2];
		extern int do_bootm(cmd_tbl_t *, int, int, char *[]);

		local_args[0] = argv[0];
		local_args[1] = NULL;

		printf("Automatic boot of image at addr 0x%08lX ...\n", addr);
		do_bootm(cmdtp, 0, 1, local_args);
		return 1;
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
U_BOOT_CMD(
	sd,  5,  1,  do_sd,
	"sd      - SD/SDHC/MMC sub-system\n",
	"info - show available devices\n"
	"sd info <dev> - show details of device <dev>\n"
	"sd reset - reset all devices\n"
	"sd reset <dev> - reset device <dev>\n"
	"sd device - show current device\n"
	"sd device <dev> - set <dev> as the current device\n"
	"sd read <blk#> <cnt> - read <cnt> blocks starting at <blk#>\n"
	"sd write <addr> <blk#> <cnt> - write <cnt> blocks starting\n"
	"    at <blk#> from memory address <addr>\n"
	"sd boot\n"
);

U_BOOT_CMD(
	sdboot,	5,	1,	do_sdboot,
	"sdboot  - boot from SD/SDHC/MMC device\n",
	"<addr> <dev> <blk#> <cnt> - boot <cnt> blocks starting\n"
	"    at block <blk#> on device <dev> at address <addr>\n"
);

#endif	/* CONFIG_CMD_SDHCI */
