/*****************************************************************************
*  Copyright 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/
/*
 * Board specific setup
 */

#include <config.h>
#include <version.h>
#include <asm/arch/bcm4760.h>

_TEXT_BASE:
       .word   TEXT_BASE       /* sdram load addr from config.mk */


/**********************************
 * initialize the watch dog
 */
       .globl hw_watchdog_activate
       .type  hw_watchdog_activate, function
hw_watchdog_activate:

       STMFD sp!, {r0-r2}

       /* Reset the watchdog block (workaround for an A0 silicon bug ) */
       ldr r0, =0xB00A4
       ldr r1, [r0]
       mov r2, #0x00000040
       bic r1, r1, r2
       str r1, [r0]

       ldr r1,=0x500000
       ldr r2,=0x0
loop:
       cmp r1, r2
       add r2, r2, #1	
       bne loop

       ldr r1, [r0]
       mov r2, #0x00000040
       orr r1, r1, r2
       str r1, [r0]


       MOV r2, #WDT_REG_BASE_ADDR

       /* unlock the dog
        *  *WDOG_LOCK = 0x1ACCE551
        */
       LDR r1, bcm4760_wdt_access_code
       STR r1, [r2, #WDT_R_WDOGLOCK_SEL]

       /* disable the dog
        *  *WDOG_CONTROL = 0
        */
       MOV r1, #0
       STR r1, [r2, #WDT_R_WDOGCONTROL_SEL]

       /* clear pending interrupt
        *  *WDOG_INTCLR = 0
        */
       STR r1, [r2, #WDT_R_WDOGINTCLR_SEL]

       /*
     * wait for the interrupt register to clear
     */
bcm4760_wdt_wait_intclr1:
    LDR r1, [r2, #WDT_R_WDOGRIS_SEL]
    ANDS r1, r1, #1
    BNE bcm4760_wdt_wait_intclr1

       /*
        * done if this is a de-activate
        */
       CMP r0, #0
       BEQ bcm4760_wdt_disable_exit

       /* reload the count
        *  *WDOG_LOAD = CONFIG_HW_WATCHDOG_TIMER_LOAD_VALUE
        */
       LDR r1, =CONFIG_HW_WATCHDOG_TIMER_LOAD_VALUE
       STR r1, [r2]

       /* enable the dog
        *  *WDOG_CONTROL = INTEN | RESEN
        */
       MOV r1, #(WDT_F_RESEN_MASK | WDT_F_INTEN_MASK)
       STR r1, [r2, #WDT_R_WDOGCONTROL_SEL]

bcm4760_wdt_disable_exit: 
       /* lock the dog
        *  *WDOG_LOCK = 0
        */
       MOV r1, #0
       STR r1, [r2, #WDT_R_WDOGLOCK_SEL]

       LDMFD sp!, {r0-r2}
       MOV     pc,     lr

/*************************************************************************
 *  hw_watchdog_reset
 *
 *      This routine is called to reset (keep alive) the watchdog timer
 *
 ************************************************************************/
       .globl hw_watchdog_reset
       .type  hw_watchdog_reset, function
hw_watchdog_reset:
       STMFD sp!, {r0-r1}
       MOV r1, #WDT_REG_BASE_ADDR

       /* unlock the dog
        *  *WDOG_LOCK = 0x1ACCE551
        */
       LDR r0, bcm4760_wdt_access_code
       STR r0, [r1, #WDT_R_WDOGLOCK_SEL]

       /* clear pending interrupt
        *  *WDOG_INTCLR = 0
        */
       MOV r0, #0
       STR r0, [r1, #WDT_R_WDOGINTCLR_SEL]

    /*
     * wait for the interrupt register to clear
     */
bcm4760_wdt_wait_intclr2:
    LDR r0, [r1, #WDT_R_WDOGRIS_SEL]
    ANDS r0, r0, #1
    BNE bcm4760_wdt_wait_intclr2

       /* lock the dog
        *  *WDOG_LOCK = 0
        */
       MOV r0, #0
       STR r0, [r1, #WDT_R_WDOGLOCK_SEL]

       LDMFD sp!, {r0-r1}
       MOV     pc,     lr


bcm4760_wdt_access_code:
       .byte 0x51,0xe5,0xcc,0x1a
