/*
 * (C) Copyright 2009 TomTom BV <http://www.tomtom.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Implementation of the flip-flop. This can store 1 bit of information
 * across a (watchdog) reset.
 */

#include <common.h>
#include <regs.h>
#include <asm/io.h>
#include <flipflop.h>

#ifdef CONFIG_SW_FLIPFLOP

#define PA_FLAG_LOCATION	0x52000000

void flipflop_set(int v)
{
	volatile unsigned long *flg = (unsigned long *)PA_FLAG_LOCATION;
	if (v) {
		*flg = 0xdeadbeef;
	} else {
		*flg = 0x0;
	}
}

int flipflop_get(void)
{
	volatile unsigned long *flg = (unsigned long *)PA_FLAG_LOCATION;
	return (*flg == 0xdeadbeef);
}

#else /* CONFIG_SW_FLIPFLOP */

static int is_setup = 0;

static void flipflop_setup(void)
{
	u32 reg;
	if (is_setup)
		return;
	/* Set GPK14 (clock) output low */
	reg = readl(GPKDAT) & (~0x4000);
	writel(reg | 0x0000, GPKDAT);
	/* Set GPK15 as input (Q output)
	   Set GPK13 as output (data)
	   Set GPK14 as output (clock) */
	reg = readl(GPKCON1) & (~0xFFF00000);
	writel(reg | 0x01100000, GPKCON1);
	is_setup = 1;
}

void flipflop_set(int v)
{
	u32 reg;

	flipflop_setup();

	/* Set GPK13 data output = activate */
	reg = readl(GPKDAT) & (~0x2000);
	writel(reg | (v ? 0x2000 : 0x0000), GPKDAT);
	
	/* Pulse GPK14 clock output */
	reg = readl(GPKDAT) & (~0x4000);
	writel(reg | 0x4000, GPKDAT);
	udelay(10);
	writel(reg | 0x0000, GPKDAT);
}

int flipflop_get(void)
{
	u32 reg;

	flipflop_setup();

	/* Read GPK15 (Q output) */
	reg = readl(GPKDAT) & 0x8000;

	return (reg != 0);
}

#endif /* CONFIG_SW_FLIPFLOP */

