/*
 * (C) Copyright 2009 TomTom BV <http://www.tomtom.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Implementation of the flip-flop. This can store 1 bit of information
 * across a (watchdog) reset.
 */

#include <common.h>
#include <regs.h>
#include <asm/io.h>
#include <flipflop.h>

#ifdef CONFIG_SW_FLIPFLOP

#define PA_FLAG_LOCATION	0x52000000

void flipflop_set(int v)
{
	volatile unsigned long *flg = (unsigned long *)PA_FLAG_LOCATION;
	if (v) {
		*flg = 0xdeadbeef;
	} else {
		*flg = 0x0;
	}
}

int flipflop_get(void)
{
	volatile unsigned long *flg = (unsigned long *)PA_FLAG_LOCATION;
	return (*flg == 0xdeadbeef);
}

#else /* CONFIG_SW_FLIPFLOP */

static int is_setup = 0;

static void flipflop_setup(void)
{
	u32 reg;
	if (is_setup)
		return;
	/* Set GPK2 (clock) output low */
	reg = readl(GPKDAT) & (~(1<<2));
	writel(reg | (0<<2), GPKDAT);
	/* Set GPM1 as input (Q output)
	   Set GPK2 as output (clock) */
	reg = readl(GPKCON0) & (~0x00000F00);
	writel(reg | 0x00000000, GPKCON0);
	reg = readl(GPMCON) & (~0x000000F0);
	writel(reg | 0x00000010, GPMCON);
	is_setup = 1;
}

void flipflop_set(int v)
{
	u32 reg;

	flipflop_setup();

	/* Pulse GPB4 clock output */
	reg = readl(GPBDAT) & (~(1<<4));
	writel(reg | (1<<4), GPBDAT);
	udelay(10);
	reg = readl(GPBDAT) & (~(1<<4));
	writel(reg | (0<<4), GPBDAT);
}

int flipflop_get(void)
{
	u32 reg;

	flipflop_setup();

	/* Read GPK15 (Q output) */
	reg = readl(GPMDAT) & (1<<1);

	return (reg != 0);
}

#endif /* CONFIG_SW_FLIPFLOP */

