/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <bootscript.h>
#include <common.h>
#include <regs.h>
#include <asm/io.h>
#include <asm/arch/bl1.h>

#define I2C_EEPROM_ADDR 0x50

DECLARE_GLOBAL_DATA_PTR;

extern int system_bootmode;	/* Bootmode from common/main.c */

/* Conservative microseconds - calibrated with I-cache enabled */
__early void early_udelay(int n)
{
	n*=200;

asm volatile (
	"1:	subs %0, %0, #1\n\t"
	"	bne 1b"
	: "+r" (n)
);

}

#define PEEKADDR 0
#define PEEKLEN  16

#ifdef CONFIG_TT_HEADER
int __early bl1_check_bricknum(void)
{
	uchar buf[PEEKLEN];
	uchar c = 0;
	int i;

	i2c_init (CFG_I2C_SPEED, CFG_I2C_SLAVE); /* Hardcoded to 100kHz */

	/* Figure out if the EEPROM has been programmed */
	if (i2c_read(I2C_EEPROM_ADDR, PEEKADDR, 1, buf, PEEKLEN)) {
		return -1;
	}

	/* Don't have memcmp in RAM yet
	 * memcmp(zeros, buf, PEEKLEN);
	 */
	if (buf[0] != 0x00 && (c=0xff) != buf[0]) {
		return -1;
	}
	for(i=1; i<PEEKLEN; i++) {
		if (buf[i] != c)
			return -1;
	}

	return 0;
}
#endif

static void __inline__ enable_sdcard(void)
{
	u32 reg;

	/* Set pin to output in GPIO CON */
	reg = readl(GPPCON) & ~0x30000;
	writel(reg | 0x1000, GPPCON);

	/* Switch off pullup/downs */
	reg = readl(GPPPUD) & ~0x30000;
	writel(reg | 0x0, GPPPUD);

	/* Set output to high */
	reg = readl(GPPDAT) & ~0x100;
	writel(reg | 0x1, GPPDAT);
}

void __early setup_board(void)
{
	u32 reg;

	/* Set I2C0_3V3_ON (=GPK8) to output in GPIO CON,
	 * Switch ADAU1361 power on, so that it lets go of the I2C lines
	 */

	reg = readl(GPKCON1) & ~0x00000f0f;
	writel(reg | 0x0000101, GPKCON1);

	reg = readl(GPKDAT) & ~0x00000100;
	writel(reg | 0x00000500, GPKDAT);

	early_udelay(100);
}

int board_init(void)
{
	char bootcmd[32];
	char ulong_str[16]; /* Needs to be as long as "0x12345678" + '\0' */

	gd->bd->bi_arch_number = MACH_TYPE_VENICE;
	gd->bd->bi_boot_params = (PHYS_SDRAM_1+0x100);

	if (readl(RST_STAT) & BIT0) {
		/* HW_RESET */
		setenv("sysboot_mode","cold");
	} else {
		setenv("sysboot_mode","watchdog");
	}

	enable_sdcard();

	IMPORT_BOOTSCRIPT(preboot_plat, preboot_plat);
#ifdef CONFIG_DEBUG_BUILD
	IMPORT_BOOTSCRIPT(preboot_debug, preboot);
	IMPORT_BOOTSCRIPT(bootcmd_debug, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd_debug, altbootcmd);
	IMPORT_BOOTSCRIPT(preboot, preboot.default);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd.default);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd.default);
#else
	IMPORT_BOOTSCRIPT(preboot, preboot);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd);
#endif

	/* Set up the environment */
	sprintf(ulong_str, "%p", CFG_LOAD_ADDR);
	setenv("loadaddr", ulong_str);

	sprintf(ulong_str, "%p", CFG_FDT_ADDR);
	setenv("fdaddr", ulong_str);

	sprintf(ulong_str, "%#x", MEMADDR_KERN_LEN);
	setenv("kernel.maxsize", ulong_str);

	setenv("kernel.console", "ttySAC3");

	return 0;
}

int dram_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE;

	return 0;
}

#ifdef BOARD_LATE_INIT
int board_late_init (void)
{
	return 0;
}
#endif

#ifdef CONFIG_DISPLAY_BOARDINFO
int checkboard(void)
{
	printf("Board:      Venice\n");
	return (0);
}
#endif

#ifdef CONFIG_ENABLE_MMU
ulong virt_to_phy_smdk6430(ulong addr)
{
	if ((0xc0000000 <= addr) && (addr < 0xc8000000))
		return (addr - 0xc0000000 + 0x50000000);
	else
		printf("do not support this address : %08lx\n", addr);

	return addr;
}
#endif

