#include <common.h>

#include "bl1_sb_6432s.h"
#include "asm/arch/bl1.h"

#define SHA1_BLOCK_LEN		64			// Input Message Block Length
#define SHA1_DIGEST_LEN		20			// Hash Code Length

/* SHA1 Context */
typedef struct {
	unsigned int	auChain[SHA1_DIGEST_LEN/4];	//	Chaining Variable
	unsigned int	auCount[2];			//	the number of input message	bit
	unsigned char	abBuffer[SHA1_BLOCK_LEN];	//	Buffer for unfilled block
} SHA1_ALG_INFO;

/* 6410/6432 version of iROM crypto routines */
struct secboot {
	int  (*memcmp)		(unsigned char*, unsigned char*, unsigned int);
	void (*memcpy)		(unsigned char*, unsigned char*, unsigned int);
	void (*memset)		(unsigned char*, unsigned int, unsigned int);
	void (*memxor)		(unsigned char*, unsigned char*, unsigned char*, unsigned int);

	int  (*sha1_init)	(SHA1_ALG_INFO*);
	int  (*sha1_update)	(SHA1_ALG_INFO*, unsigned char*, unsigned int);
	int  (*sha1_final)	(SHA1_ALG_INFO*, unsigned char*);

	int  (*rsa_pss_verify)	(unsigned char*, int, unsigned char*, int, unsigned char*, int);
} const def_secboot __early_data = {
	0x00002bf0,
	0x00002c38,
	0x00002d30,
	0x00002da4,
	0x000027e4,
	0x0000282c,
	0x000028fc,
	0x000022c0,
};

#include "rsa_pubkey.h"

#define SRAM_BASE 0x0c000000
#define ADDR_IN_SRAM(x)	(((uchar*)x) - TEXT_BASE + SRAM_BASE)

/* Verify integrity of BL2(or OS) Image. */
int __early Check_IntegrityOfBL2( SecureBoot_CTX *sbContext, unsigned char *BL2, int BL2Len,
	unsigned char *BL2_SignedData, int BL2_SignedDataLen )
{
	unsigned int	i, rv;
	unsigned char	c, hashCode[SHA1_DIGEST_LEN];

	SHA1_ALG_INFO	algInfo;

	struct secboot *sb = (struct secboot*)sbContext->func_ptr_BaseAddr;
	if (sb->sha1_init == NULL || sb->sha1_init >= NULL+0x8000 ||
	    sb->rsa_pss_verify == NULL || sb->rsa_pss_verify >= NULL+0x8000)
		sb = ADDR_IN_SRAM(&def_secboot);

	// 1. Make HashCode
	// 1-1. SHA1 Init
	if (SB_OK != (rv = sb->sha1_init(&algInfo)))
		return rv;

	// 1-3. SHA1 Update.
	if (SB_OK != (rv = sb->sha1_update(&algInfo, BL2, BL2Len )))
		return rv;

	// 1-4. SHA1 Final.
	if (SB_OK != (rv = sb->sha1_final(&algInfo, hashCode )))
		return rv;

	// 2. BL2's signature verification
	if (SB_OK != (rv = sb->rsa_pss_verify((void*)ADDR_IN_SRAM(&BL2_Key), sizeof(RawRSAPublicKey),
			hashCode, SHA1_DIGEST_LEN, BL2_SignedData, BL2_SignedDataLen )))
		return rv;
	
	return SB_OK;
}

