/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 * Gary Jennejohn <gj@denx.de>
 * David Mueller <d.mueller@elsoft.ch>
 *
 * Configuation settings for the SAMSUNG SMDK6400(mDirac-III) board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#define CONFIG_TT_LIBFDT

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_S5P6440		1		/* in a SAMSUNG S5P6440 SoC */
#define CONFIG_S5P64XX		1		/* in a SAMSUNG S5P64XX Family  */
#define CONFIG_SMDK6440		1		/* on a SAMSUNG SMDK6440 Board  */

#define MEMORY_BASE_ADDRESS	(0x20000000)

/* input clock of PLL */
#define CONFIG_SYS_CLK_FREQ	12000000	/* the SMDK6400 has 12MHz input clock */

#define CONFIG_ENABLE_MMU
#ifdef CONFIG_ENABLE_MMU
#define virt_to_phys(x)	virt_to_phy_smdk6440(x)
#else
#define virt_to_phys(x)	(x)
#endif

#define CONFIG_MEMORY_UPPER_CODE

#undef CONFIG_USE_IRQ				/* we don't need IRQ/FIQ stuff */

#define CONFIG_INCLUDE_TEST

#define CONFIG_ZIMAGE_BOOT
#define CONFIG_IMAGE_BOOT

#define BOARD_LATE_INIT

#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_CMDLINE_TAG
#define CONFIG_INITRD_TAG

#define CONFIG_HW_WATCHDOG

/*
 * Architecture magic and machine type
 */
#define MACH_TYPE		2032
#define UBOOT_MAGIC		(0x43090000 | MACH_TYPE)

/* Power Management is enabled */
#define CONFIG_PM

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO

#undef CONFIG_SKIP_RELOCATE_UBOOT
#undef CONFIG_USE_NOR_BOOT

/*
 * Size of malloc() pool
 */
#define CFG_MALLOC_LEN		(CFG_ENV_SIZE + 1024*1024)
#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */

#define CFG_STACK_SIZE		512*1024

/*
 * select serial console configuration
 */
#define CONFIG_SERIAL3          1	/* we use UART2 (aka SERIAL3) on Cordoba */

#define CFG_HUSH_PARSER			/* use "hush" command parser	*/
#ifdef CFG_HUSH_PARSER
#define CFG_PROMPT_HUSH_PS2	"> "
#endif

#define CONFIG_CMDLINE_EDITING

#undef CONFIG_S5P64XX_I2C		/* this board has H/W I2C */
#ifdef CONFIG_S5P64XX_I2C
#define CONFIG_HARD_I2C		1
#define CFG_I2C_SPEED		50000
#define CFG_I2C_SLAVE		0xFE
#endif

#define CONFIG_DOS_PARTITION
#define CONFIG_SUPPORT_VFAT

#define CONFIG_USB_OHCI
#undef CONFIG_USB_STORAGE
#undef CONFIG_S3C_USBD

#define USBD_DOWN_ADDR		(0xc0000000)

/************************************************************
 * RTC
 ************************************************************/
#define CONFIG_RTC_S5P64XX	1

/* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE

#define CONFIG_BAUDRATE		115200

/******************************************************************************
 * Boot sequence scripts - these scripts are run by U-Boot to boot the kernel *
 ******************************************************************************/
#define _MK_STRING(x)	#x
#define MK_STRING(x)	_MK_STRING(x)


/* Board specific memory map used in U-Boot */
#define TT_BOARD_KLOADADDR      0xC0008000      /* Address to load kernel zImage */
#define TT_BOARD_KIMGNAME       "zImage"        /* Name of kernel zImage on disk */
#define TT_BOARD_CMDLOADADDR    0xC0000000      /* Address to load cmdline.txt */
#define CMDLINE_TXT_MAXLEN      0x800           /* Max length thta cmdline.txt may have */
#define TT_BOARD_KTAGADDR       0xC0000800      /* Address to load ATAG kernel params */
#define TT_BOARD_FDADDR		0x22001000	/* Address to load factory data device-tree (MUST be 4k-page-aligned) */

/* Command "boot" or "run bootcmd" does normal boot sequence */
#define SCR_BOOT_NORMAL                                         \
        "echo *** Version using device-tree and watchdog; "     \
        "setenv verify n; "                                     \
        "setenv fd_addr " MK_STRING(TT_BOARD_FDADDR) "; "       \
        "getbootmode tt_sysboot_mode; "                         \
        "getbootfail tt_sysboot_fail; "                         \
        "watchdog off; setenv tt_activate_wdt 0; "              \
        "if test \"${tt_sysboot_fail}:${tt_sysboot_mode}\" = "  \
               "\"fail:watchdog\" ; "                           \
        "then; "                                                \
          "echo *** Last boot fail ***; "                       \
          "getpart tt_bootpart ext2 mmc 1; "                    \
          "if test 0 -ne $?; "                                  \
          "then; "                                              \
            "echo Could not find backup root; "                 \
            "exit 1; "                                          \
          "fi; "                                                \
        "else; "                                                \
          "echo *** Last boot pass ***; "                       \
          "getpart tt_bootpart boot mmc 1; "                    \
          "if test 0 -ne $?; "                                  \
          "then; "                                              \
            "echo Could not find partition marked boot, trying "\
               "backup root fs; "                               \
            "getpart tt_bootpart ext2 mmc 1; "                  \
            "if test 0 -ne $?; "                                \
            "then; "                                            \
              "echo Could not find backup root; "               \
              "exit 1; "                                        \
            "fi; "                                              \
          "else; "                                              \
            "getpart tt_backuppart ext2 mmc 1; "                \
            "if test ${tt_backuppart} -ne ${tt_bootpart}; "     \
            "then; "                                            \
/*              "echo *** Activating watchdog ***; "              \*/\ 
/*              "watchdog on; setenv tt_activate_wdt 1; "         \*/\
              "echo *** NOT Activating watchdog ***; "          \
            "fi; "                                              \
          "fi; "                                                \
        "fi; "                                                  \
        "setenv tt_fatpart 0; "                                 \
        "getpart tt_fatpart fat mmc 1 && "                      \
        "getpart tt_fdpart fd mmc 1; "                          \
        "fdload mmc 1:${tt_fdpart} ${fd_addr}; "                \
        "echo *** Loading kernel from mmc ***; "                \
        "ext2load mmc 1:${tt_bootpart} "                        \
         MK_STRING(TT_BOARD_KLOADADDR) " "                      \
         TT_BOARD_KIMGNAME "; "                                 \
        "echo *** Loaded kernel from mmc ***; "                 \
        "mw.b " MK_STRING(TT_BOARD_CMDLOADADDR) " 0x0 "         \
         MK_STRING(CMDLINE_TXT_MAXLEN) "; "                     \
        "if fatload mmc 1:${tt_fatpart} "                       \
         MK_STRING(TT_BOARD_CMDLOADADDR) " cmdline.txt; "       \
        "then; "                                                \
           "setenvram cmdline_args " MK_STRING(TT_BOARD_CMDLOADADDR)\
             " " MK_STRING(CMDLINE_TXT_MAXLEN) "; "             \
	   "setenv bootargs \"${bootargs} ${cmdline_args}\"; "  \
           "echo Using cmdline.txt; "                           \
        "else; "                                                \
           "echo; "                                             \
           "echo cmdline.txt not found; "                       \
        "fi; "                                                  \
        "echo bootargs=${bootargs}; "                           \
        "echo; "                                                \
        "bootm " MK_STRING(TT_BOARD_KLOADADDR)

#define CONFIG_BOOTCOMMAND SCR_BOOT_NORMAL

/* Command "run bootcmd_backup" boots linux from  backup root partition */
#define SCR_BOOT_BACKUP                                         \
"bootcmd_backup="                                               \
        "setenv fd_addr " MK_STRING(TT_BOARD_FDADDR) "; "       \
        "getpart tt_bootpart ext2 mmc 1; "                      \
        "if test 0 -ne $?; "                                    \
        "then; "                                                \
        "   echo Could not find backup root; "                  \
        "   exit 1; "                                           \
        "fi; "                                                  \
        "getbootmode tt_sysboot_mode; "                         \
        "getpart tt_fdpart fd mmc 1; "                          \
        "fdload mmc 1:${tt_fdpart} ${fd_addr}; "                \
        "ext2load mmc 1:${tt_bootpart} "                        \
         MK_STRING(TT_BOARD_KLOADADDR) " "                      \
         TT_BOARD_KIMGNAME "; "                                 \
        "echo bootargs=${bootargs}; "                           \
        "echo; "                                                \
        "bootm " MK_STRING(TT_BOARD_KLOADADDR)

/* Add extra boot commands to environment */
#define EXTRA_COMMANDS                  \
        SCR_BOOT_BACKUP         "\0"

/******************************************************************************
 * Boot arguments - these args are supplied to the kernel unless cmdline.txt  *
 *                  is found.                                                 *
 ******************************************************************************/

#define BOOTARGS_BASE   \
        "root=/dev/mmcblk0p${tt_bootpart} ro "  \
        "rootwait mem=64M "     \
        "console=ttySAC2,115200 "       \
        "sysboot_mode=${tt_sysboot_mode} " \
        "s3c2410_wdt.tmr_atboot=${tt_activate_wdt} " \
	"lpj=" MK_STRING(PRE_CALIBRATED_LOOPS_PER_JIFFY)

#ifdef PANEL_TOSHIBA_WVGA
# define CONFIG_BOOTARGS BOOTARGS_BASE          \
        "panel=3 bpp=24 "
#elif defined(PANEL_SAMSUNG_WVGA)
# define CONFIG_BOOTARGS BOOTARGS_BASE          \
        "panel=2 bpp=24 "
#else /* No panel */
# define CONFIG_BOOTARGS BOOTARGS_BASE
#endif


/***********************************************************
 * Command definition
 ***********************************************************/

#define CONFIG_CMD_BDI          /* bdinfo                       */
#define CONFIG_CMD_BOOTD        /* bootd                        */
#define CONFIG_CMD_CONSOLE      /* coninfo                      */
#define CONFIG_CMD_ECHO         /* echo arguments               */
#define CONFIG_CMD_ENV          /* saveenv                      */
#define CONFIG_CMD_FLASH        /* flinfo, erase, protect       */
#define CONFIG_CMD_FPGA         /* FPGA configuration Support   */
#define CONFIG_CMD_IMI          /* iminfo                       */
#define CONFIG_CMD_IMLS         /* List all found images        */
#define CONFIG_CMD_ITEST        /* Integer (and string) test    */
#define CONFIG_CMD_LOADB        /* loadb                        */
#define CONFIG_CMD_LOADS        /* loads                        */
#define CONFIG_CMD_MEMORY       /* md mm nm mw cp cmp crc base loop mtest */
#define CONFIG_CMD_MISC         /* Misc functions like sleep etc*/
#define CONFIG_CMD_NET          /* bootp, tftpboot, rarpboot    */
#define CONFIG_CMD_NFS          /* NFS support                  */
#define CONFIG_CMD_RUN          /* run command in env variable  */
#define CONFIG_CMD_SETGETDCR    /* DCR support on 4xx           */
#define CONFIG_CMD_FAT          /**/

/* TomTom specific commands */
#define CONFIG_CMD_PART         /* getpart command */
#define CONFIG_CMD_FD           /* fdload  command */
#define CONFIG_CMD_SETENVRAM

#define ENABLE_DCHACE_FOR_BOOTM_CMD


#define CONFIG_CMD_CACHE
#define CONFIG_CMD_USB
#define CONFIG_CMD_REGINFO
#define CONFIG_CMD_MTD
#define	CONFIG_CMD_NAND

//#define CONFIG_CMD_ONENAND
#define CONFIG_CMD_MOVINAND
#define CONFIG_CMD_PING
#define CONFIG_CMD_DATE
#define CONFIG_CMD_EXT2
#define CONFIG_CMD_NET

#define CONFIG_CMD_ENV
#define CONFIG_CMD_DFL

#include <config_cmd_default.h>

#define CONFIG_CMD_ELF
#define CONFIG_CMD_DHCP
#undef CONFIG_CMD_I2C

#undef CONFIG_CMD_WATCHDOG
/*
 * BOOTP options
 */
#define CONFIG_BOOTP_SUBNETMASK
#define CONFIG_BOOTP_GATEWAY
#define CONFIG_BOOTP_HOSTNAME
#define CONFIG_BOOTP_BOOTPATH

#define CONFIG_BOOTDELAY	3
#undef CONFIG_SILENT_CONSOLE

//#define CONFIG_BOOTARGS    	"root=ramfs devfs=mount console=ttySA0,9600"
#define CONFIG_ETHADDR		00:40:5c:26:0a:5b
#define CONFIG_NETMASK          255.255.255.0
#define CONFIG_IPADDR		192.168.0.20
#define CONFIG_SERVERIP		192.168.0.10
#define CONFIG_GATEWAYIP	192.168.0.1

#define CONFIG_ZERO_BOOTDELAY_CHECK

//#define CONFIG_NET_MULTI

#ifdef CONFIG_NET_MULTI
#define CONFIG_DRIVER_SMC911X
#define CONFIG_DRIVER_SMC911X_BASE	(0x18800300)
#define CONFIG_DRIVER_SMC911X_16_BIT
#define CONFIG_DRIVER_CS8900
#define CS8900_BASE	  		(0x18800300)
#define CS8900_BUS16

#else

#define CONFIG_DRIVER_DM9000	1

#ifdef 	CONFIG_DRIVER_SMC911X
#define CONFIG_DRIVER_SMC911X_16_BIT
#undef	CONFIG_DRIVER_CS8900
#define CONFIG_DRIVER_SMC911X_BASE	(0x18800300)
#endif

#ifdef CONFIG_DRIVER_CS8900
#define CS8900_BASE	  		(0x18800300)
#define CS8900_BUS16
#endif

#ifdef CONFIG_DRIVER_DM9000
#define CONFIG_DM9000_BASE		(0x80000000)
#define DM9000_IO			(CONFIG_DM9000_BASE)
#define DM9000_DATA			(CONFIG_DM9000_BASE+2)
#endif

#endif

#if defined(CONFIG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	115200		/* speed to run kgdb serial port */
/* what's this ? it's not used anywhere */
#define CONFIG_KGDB_SER_INDEX	1		/* which serial port to use */
#endif

/* TomTom factory data custom ATAG option */
#define CONFIG_FACTORYDATA_TAG


/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP				/* undef to save memory		*/
#define CFG_PROMPT		"SMDK6440 # "	/* Monitor Command Prompt	*/
#define CFG_CBSIZE		256		/* Console I/O Buffer Size	*/
#define CFG_PBSIZE		384		/* Print Buffer Size */
#define CFG_MAXARGS		16		/* max number of command args	*/
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size	*/

#define CFG_MEMTEST_START	(MEMORY_BASE_ADDRESS)	/* memtest works on	*/
#define CFG_MEMTEST_END		(MEMORY_BASE_ADDRESS + 0x7e00000)	/* 128 MB in DRAM */

#undef CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */

#define CFG_LOAD_ADDR		MEMORY_BASE_ADDRESS	/* default load address	*/

/* the PWM TImer 4 uses a counter of 15625 for 10 ms, so we need */
/* it to wrap 100 times (total 1562500) to get 1 sec. */
#define CFG_HZ			1562500		/* at PCLK 50MHz */

/* valid baudrates */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	0x40000		/* regular stack 256KB */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(4*1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4*1024)	/* FIQ stack */
#endif

//#define CONFIG_CLK_667_166_83
#define CONFIG_CLK_533_133_66
//#define CONFIG_CLK_400_100_50

#define set_pll(mdiv, pdiv, sdiv)	(1<<31 | mdiv<<16 | pdiv<<8 | sdiv)

#if defined(CONFIG_CLK_667_166_83)
#define APLL_MDIV		667
#define APLL_PDIV		6
#define APLL_SDIV		1
#elif defined(CONFIG_CLK_533_133_66)
#define APLL_MDIV		533
#define APLL_PDIV		6
#define APLL_SDIV		1
#elif defined(CONFIG_CLK_400_100_50)
#define APLL_MDIV		400
#define APLL_PDIV		3
#define APLL_SDIV		2
#else
#error "Not Support Fequency or Mode!! you have to setup right configuration."
#endif

#define MPLL_MDIV		260
#define MPLL_PDIV		2
#define MPLL_SDIV		2

#define APLL_VAL	set_pll(APLL_MDIV, APLL_PDIV, APLL_SDIV)
#define Startup_APLL	(CONFIG_SYS_CLK_FREQ/(APLL_PDIV<<APLL_SDIV)*APLL_MDIV)

#define MPLL_VAL	set_pll(MPLL_MDIV, MPLL_PDIV, MPLL_SDIV)
#define Startup_MPLL	((CONFIG_SYS_CLK_FREQ)/(MPLL_PDIV<<MPLL_SDIV)*MPLL_MDIV)

#if defined(CONFIG_CLK_667_166_83)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		3
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#elif defined(CONFIG_CLK_533_133_66)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		3
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#elif defined(CONFIG_CLK_400_100_50)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		3
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#else
#error "Not Support Fequency or Mode!! you have to setup right configuration."
#endif

#define CLK_DIV_VAL	((Startup_PCLKdiv<<12)|(Startup_HCLKdiv<<8)|(Startup_MPLLdiv<<4)|Startup_ARMdiv)

#define Startup_HCLK	(Startup_APLL/(Startup_HCLKx2div+1)/(Startup_HCLKdiv+1))

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#ifndef CONFIG_SMDK6430_X5A

#define DMC1_MEM_CFG		0x00010012	/* Supports one CKE control, Chip1, Burst4, Row/Column bit */
#define DMC1_MEM_CFG2		0xB45
#define DMC1_CHIP0_CFG		0x150F8
#define DMC_DDR_32_CFG		0x0 		/* 32bit, DDR */

/* Memory Parameters */
/* DDR Parameters */
#define DDR_tREFRESH		7800		/* ns */
#define DDR_tRAS		45		/* ns (min: 45ns)*/
#define DDR_tRC 		68		/* ns (min: 67.5ns)*/
#define DDR_tRCD		23		/* ns (min: 22.5ns)*/
#define DDR_tRFC		80		/* ns (min: 80ns)*/
#define DDR_tRP 		23		/* ns (min: 22.5ns)*/
#define DDR_tRRD		15		/* ns (min: 15ns)*/
#define DDR_tWR 		15		/* ns (min: 15ns)*/
#define DDR_tXSR		120		/* ns (min: 120ns)*/
#define DDR_CASL		3		/* CAS Latency 3 */

#else

#define DMC1_MEM_CFG		0x00210011	/* Supports one CKE control, Chip1, Burst4, Row/Column bit */
#define DMC1_MEM_CFG2		0xB41
#define DMC1_CHIP0_CFG		0x150FC
#define DMC1_CHIP1_CFG		0x154FC
#define DMC_DDR_32_CFG		0x0		/* 32bit, DDR */

/* Memory Parameters */
/* DDR Parameters */
#define DDR_tREFRESH		5865		/* ns */
#define DDR_tRAS		50		/* ns (min: 45ns)*/
#define DDR_tRC 		68		/* ns (min: 67.5ns)*/
#define DDR_tRCD		23		/* ns (min: 22.5ns)*/
#define DDR_tRFC		133		/* ns (min: 80ns)*/
#define DDR_tRP 		23		/* ns (min: 22.5ns)*/
#define DDR_tRRD		20		/* ns (min: 15ns)*/
#define DDR_tWR 		20		/* ns (min: 15ns)*/
#define DDR_tXSR		125		/* ns (min: 120ns)*/
#define DDR_CASL		3		/* CAS Latency 3 */

#endif

/*
 * mDDR memory configuration
 */
#define DDR_PHYCONTROL0		0x6A101003	
#define DDR_PHYCONTROL1		0x000084F4
#define DDR_PHYCONTROL2		0x00000000
#define DDR_CONCONTROL_DIS	0x0FFF1010
#define DDR_MEMCONTROL		0x00202100
#define DDR_PRECHCONTROL	0x20000000
#define TIMING_AREF 		0x0000040D
#define DDR_TIMINGROW		0x0A233246
#define DDR_TIMINGDATA		0x12130003
#define DDR_TIMINGPOWER 	0x0E100122
#define DDR_DIRECTCMD_NOP	0x07000000
#define DDR_DIRECTCMD_PALL	0x01000000
#define DDR_DIRECTCMD_REFA	0x05000000
#define DDR_SET_MRS		0x00000032
#define DDR_SET_EMRS 		0x00020000
#define DDR_CONCONTROL_EN 	0x0FFF10B0
#define DMC_DDR_PWRDNCONFIG 	0x00100004

#define CONFIG_NR_DRAM_BANKS	1	   /* we have 2 bank of DRAM */
#define PHYS_SDRAM_1		MEMORY_BASE_ADDRESS /* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE	0x10000000 /* 256 MB */

#define CFG_FLASH_BASE		(0x00000000)

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#define CFG_MAX_FLASH_BANKS	0	/* max number of memory banks */
#define CFG_MAX_FLASH_SECT	1024
#define CONFIG_AMD_LV800
#define PHYS_FLASH_SIZE		0x100000

/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT	(5*CFG_HZ) /* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(5*CFG_HZ) /* Timeout for Flash Write */

#define CFG_ENV_ADDR		0
#define CFG_ENV_SIZE		0x4000	/* Total Size of Environment Sector */

/*
 * SMDK6400 board specific data
 */

#define CONFIG_IDENT_STRING	"for SMDK6440"

/* total memory required by uboot */
#define CFG_UBOOT_SIZE		(2*1024*1024)

/* base address for uboot */
#ifdef CONFIG_ENABLE_MMU
#define CFG_UBOOT_BASE		(0xc7e00000)
#else
#define CFG_UBOOT_BASE		(0x27e00000)
#endif
#define CFG_PHY_UBOOT_BASE	(MEMORY_BASE_ADDRESS + 0x7e00000)

#define CFG_ENV_OFFSET		0x0007c000

#define CFG_MAX_NAND_DEVICE     1
#define CFG_NAND_BASE           (0x70200010)
#define CFG_NAND_SKIP_BAD_DOT_I	1  /* ".i" read skips bad blocks   */
#define	CFG_NAND_WP		1
#define CFG_NAND_YAFFS_WRITE	1  /* support yaffs write */
#define CFG_NAND_HWECC
#undef	CFG_NAND_FLASH_BBT
#define CONFIG_NAND_BL1_8BIT_ECC
#define NAND_MAX_CHIPS          1
#define NAND_DISABLE_CE()	(NFCONT_REG |= (1 << 1))
#define NAND_ENABLE_CE()	(NFCONT_REG &= ~(1 << 1))
#define NF_TRANSRnB()		do { while(!(NFSTAT_REG & (1 << 0))); } while(0)
#define CONFIG_MMC
#define CONFIG_MTDPARTITION	"40000 3c0000 3000000"
#define CFG_ONENAND_BASE 	(0x70100000)
#define CFG_MAX_ONENAND_DEVICE	1
#define CFG_ENV_IS_IN_AUTO

#endif	/* __CONFIG_H */

