/*
 * unrandom.h - pseudo random number generator
 *
 * 2009-11-05 Ard Biesheuvel <ard.biesheuvel@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

/*
 * This code uses the ARM cycle counter as a source of pseudo-randomness.
 * The init() function should be called as early as possible to start the counter.
 * The get() function reads the counter, applies sha1() to its value to expand it
 * from 32 bits to 160 bits, and disables the counter again.
 */
#include <sha1.h>

#define UNRANDOM_SIZE		20

#define UNRANDOM_INIT()		asm("mcr p15, 0, %0, c15, c12, 0"::"r"(1))

#define UNRANDOM_GET(buf)	do {							\
					unsigned long l; 				\
					asm("mrc p15, 0, %0, c15, c12, 1":"=r"(l));	\
					sha1_csum( (void*)&l, sizeof(l), buf );		\
				} while (0)

#define UNRANDOM_FINI()		asm("mcr p15, 0, %0, c15, c12, 0"::"r"(0))

